module Tins
  require 'tins/attempt'
  require 'tins/bijection'
  require 'tins/count_by'
  require 'tins/deep_dup'
  require 'tins/file_binary'
  require 'tins/find'
  require 'tins/generator'
  require 'tins/go'
  require 'tins/hash_symbolize_keys_recursive'
  require 'tins/hash_union'
  require 'tins/limited'
  require 'tins/lines_file'
  require 'tins/memoize'
  require 'tins/minimize'
  require 'tins/module_group'
  require 'tins/null'
  require 'tins/once'
  require 'tins/p'
  require 'tins/partial_application'
  require 'tins/range_plus'
  require 'tins/require_maybe'
  require 'tins/round'
  require 'tins/secure_write'
  require 'tins/shuffle'
  require 'tins/string_camelize'
  require 'tins/string_underscore'
  require 'tins/string_version'
  require 'tins/subhash'
  require 'tins/time_dummy'
  require 'tins/date_dummy'
  require 'tins/date_time_dummy'
  require 'tins/time_freezer'
  require 'tins/to_proc'
  require 'tins/uniq_by'
  require 'tins/version'
  require 'tins/write'
  require 'tins/extract_last_argument_options'
  require 'tins/deep_const_get'
  require 'tins/responding'
  require 'tins/proc_compose'
  require 'tins/proc_prelude'
  require 'tins/concern'
  require 'tins/rotate'
  require 'tins/to'
  require 'tins/terminal'
  require 'tins/sexy_singleton'
  require 'tins/method_description'
  require 'tins/annotate'
  require 'tins/token'
  require 'tins/dslkit'
  if defined? ::Encoding
    require 'tins/string_byte_order_mark'
  end
end
require 'tins/alias'
